// ********************************************************************
// 	
// 	Project	: Dynamic Graphic User Interface (DGUI)
// 	Author	: Gamza
// 	Mail	: Gamza@hitel.net
// 	Home	: http://www.Gamza.net
// 	copyright	:
// 	      ,, .
// 	     
// 	     		 ø...@~
// 	
// *******************************************************************

#ifndef INPUTSYSTEM_H_HEADER_INCLUDED
#define INPUTSYSTEM_H_HEADER_INCLUDED

// *******************************************************************
// Է»¸ ϱ  ü.
// *******************************************************************
// Ѱ 콺ġ 128 Ű ¸   .
//  콺 ư  ʰ, 0~31 Ű 콺 ư .
// ڰ ־ϴ Ű ¸ ָ ȴ.
class UserInputState
{
public:
	UserInputState(){Clear();}
	inline void Clear()
	{
		m_MousePos[0]	= 0;
		m_MousePos[1]	= 0;
		m_KeyState[0]	= 0;
		m_KeyState[1]	= 0;
		m_KeyState[2]	= 0;
		m_KeyState[3]	= 0;
	}
	inline void SetMousePos( long x, long y )
	{
		m_MousePos[0] = x;
		m_MousePos[1] = y;
	}
	inline void SetMouseButton( long keycode )
	{
		//assert( keycode>=0 );
		//assert( keycode<32 );
		//assert  SetKey .
		SetKey( keycode );
	}
	inline void SetKey( long keycode )
	{
		//assert( keycode>=0 );
		//assert( keycode<128 );
		m_KeyState[((keycode>>5)&0x3)] |= (1<<(keycode&0x1f));
	}
	inline void GetMousePos( long *px, long *py ) const
	{
		*px = m_MousePos[0];
		*py = m_MousePos[1];
	}
	inline bool GetMouseButton( long keycode ) const
	{
		//assert( keycode>=0 );
		//assert( keycode<32 );
		//assert  GetKey .
		return GetKey( keycode );
	}	
	inline bool GetKey( long keycode ) const
	{
		//assert( keycode>=0 );
		//assert( keycode<128 );
		return ( m_KeyState[((keycode>>5)&0x3)] & (1<<(keycode&0x1f)) ) != 0;
	}
	inline bool CheckMousePos() const
	{
		return (0!=m_MousePos[0]) || (0!=m_MousePos[1]);
	}
	inline bool CheckMouseButton() const
	{
		return (0!=m_KeyState[0]);
	}
	inline bool CheckKeyboard() const
	{
		return (0!=m_KeyState[1]) || (0!=m_KeyState[2]) || (0!=m_KeyState[3]);
	}
	inline UserInputState operator ^( const UserInputState& state )
	{
		UserInputState result;
		result.m_MousePos[0]	= m_MousePos[0]^state.m_MousePos[0];
		result.m_MousePos[1]	= m_MousePos[1]^state.m_MousePos[1];
		result.m_KeyState[0]	= m_KeyState[0]^state.m_KeyState[0];
		result.m_KeyState[1]	= m_KeyState[1]^state.m_KeyState[1];
		result.m_KeyState[2]	= m_KeyState[2]^state.m_KeyState[2];
		result.m_KeyState[3]	= m_KeyState[3]^state.m_KeyState[3];
		return result;
	}
	inline UserInputState operator &( const UserInputState& state )
	{
		UserInputState result;
		result.m_MousePos[0]	= m_MousePos[0]&state.m_MousePos[0];
		result.m_MousePos[1]	= m_MousePos[1]&state.m_MousePos[1];
		result.m_KeyState[0]	= m_KeyState[0]&state.m_KeyState[0];
		result.m_KeyState[1]	= m_KeyState[1]&state.m_KeyState[1];
		result.m_KeyState[2]	= m_KeyState[2]&state.m_KeyState[2];
		result.m_KeyState[3]	= m_KeyState[3]&state.m_KeyState[3];
		return result;
	}

protected:
	long m_MousePos[2];
	unsigned long m_KeyState[4];
	//콺   KeyState  , 
	//콺 ư ŰŰ ID  аϰ ϱ.
	//
	//ũƮ '콺 ư 1', 'Ű ư 1'
	//̶ ϱ⺸ ׳ 'ư 1', 'ư 33'...
	//̷ ϴ° ٰ Ǳ .
};


// *******************************************************************
// ....ּ 󰥰...
// *******************************************************************
class UserInput
{
public:
	UserInput(){}
	virtual ~UserInput(){}
	// *******************************************************************
	// 1.   Ҷ  Clear Ѵ. ( Clear ȣ )
	// *******************************************************************
	inline void Clear(){ m_CurState.Clear(); }
	// *******************************************************************
	// 2. ִ Է ¸ ִ´.
	// *******************************************************************
	inline void SetMousePos( long x, long y )	{ m_CurState.SetMousePos( x,y ); }
	inline void SetMouseButton( long keycode )	{ m_CurState.SetMouseButton( keycode ); }
	inline void SetKey( long keycode )			{ m_CurState.SetKey( keycode ); }
	// *******************************************************************
	// 3. ̺Ʈ ˻Ѵ. ( CheckEvent ȣ )
	// *******************************************************************	
	inline bool CheckEvent()
	{		
		m_Event		= m_CurState ^ m_OldState;
		m_Press		= m_Event & m_CurState;
		m_Release	= m_Event & m_OldState;
		m_OldState = m_CurState;
		// ƹ ̺Ʈ Ͼ ʾҴٸ false ȯѴ.
		return IsMouseEvent() || IsKeyboardEvent();
	}
	inline bool IsMouseEvent()				const {	return IsMouseMoveEvent() || IsMouseButtonEvent(); }
	inline bool IsKeyboardEvent()			const {	return m_Event.CheckKeyboard(); }
	inline bool IsKeyboardPressEvent()		const {	return m_Press.CheckKeyboard(); }
	inline bool IsKeyboardReleaseEvent()	const {	return m_Release.CheckKeyboard(); }
	inline bool IsMouseMoveEvent()			const {	return m_Event.CheckMousePos(); }
	inline bool IsMouseButtonEvent()		const {	return m_Event.CheckMouseButton(); }
	inline bool IsMouseButtonPressEvent()	const {	return m_Press.CheckMouseButton(); }
	inline bool IsMouseButtonReleaseEvent()	const {	return m_Release.CheckMouseButton(); }
	// *******************************************************************
	// 4. ̺Ʈ ִٸ ־ϴ Է ¸  Ȯؼ óѴ.
	// *******************************************************************
	inline void GetMousePos( long *px, long *py )		const { m_CurState.GetMousePos( px,py ); }
	inline bool GetMouseButton( long keycode )			const { return m_CurState.GetMouseButton( keycode ); }
	inline bool GetMouseButtonPress( long keycode )		const { return m_Press.GetMouseButton( keycode ); }
	inline bool GetMouseButtonRelease( long keycode )	const { return m_Release.GetMouseButton( keycode ); }
	inline bool GetKey( long keycode )					const { return m_CurState.GetKey( keycode ); }
	inline bool GetKeyPress( long keycode )				const { return m_Press.GetKey( keycode ); }
	inline bool GetKeyRelease( long keycode )			const { return m_Release.GetKey( keycode ); }

protected:
	UserInputState m_CurState;	// current input state
	UserInputState m_OldState;
	UserInputState m_Event;		// key press/release or mouse move event
	UserInputState m_Press;		// key press event
	UserInputState m_Release;	// key release event
};

#endif /* DGUI_H_HEADER_INCLUDED_C2AA8524 */
